@@ -25,42 +25,42 @@
 #include "view.h"
 #include "searchengine.h"
 #include "fontdialog.h"
-#include "prefs.h"
+#include "khc_debug.h"
+#include "navigator.h"
+#include "grantleeformatter.h"
+#include "bookmarkowner.h"
 
 #include <QAction>
 #include <KActionCollection>
 #include <KConfig>
 #include <QIcon>
-#include <KIconLoader>
 #include <KRun>
-#include <KAboutData>
 #include <KHTMLView>
 #include <KHTMLSettings>
-#include <KStandardShortcut>
 #include <QDialog>
 #include <KStandardAction>
-#include <KXmlGuiWindow>
 #include <KStartupInfo>
 #include <KConfigGroup>
 #include <KWindowConfig>
+#include <KBookmarkManager>
+#include <KBookmarkMenu>
+#include <KActionMenu>
 
 #include <QtDBus/QDBusConnection>
 #include <QSplitter>
 #include <QTextEdit>
-#include <QLayout>
 #include <QVBoxLayout>
-#include <QFrame>
 #include <QList>
 #include <QMimeDatabase>
-#include <QLoggingCategory>
-#include <QBoxLayout>
 #include <QStatusBar>
+#include <QDir>
 
-#include <stdlib.h>
 #include <QDialogButtonBox>
 #include <QPushButton>
 
-QLoggingCategory category("org.kde.khelpcenter");
+#include <kio/job.h>
+
+#include <prefs.h>
 
 using namespace KHC;
 
@@ -71,26 +71,18 @@ class LogDialog : public QDialog
       : QDialog( parent )
     {
       setWindowTitle( i18n("Search Error Log") );
-      QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok);
-      QVBoxLayout *mainLayout = new QVBoxLayout;
-      setLayout(mainLayout);
-      QPushButton *okButton = buttonBox->button(QDialogButtonBox::Ok);
-      okButton->setDefault(true);
-      okButton->setShortcut(Qt::CTRL | Qt::Key_Return);
-      connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
-      connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 
-      QFrame *topFrame = new QFrame( this );
-      mainLayout->addWidget(topFrame);
-      mainLayout->addWidget(buttonBox);
+      QVBoxLayout *mainLayout = new QVBoxLayout( this );
 
-      QBoxLayout *topLayout = new QVBoxLayout( topFrame );
-
-      mTextView = new QTextEdit( topFrame );
-      mTextView->setReadOnly ( true );
+      mTextView = new QTextEdit( this );
+      mTextView->setReadOnly( true );
       mTextView->setWordWrapMode( QTextOption::NoWrap );
-      topLayout->addWidget( mTextView );
-      
+      mainLayout->addWidget( mTextView );
+
+      QDialogButtonBox *buttonBox = new QDialogButtonBox( QDialogButtonBox::Close );
+      connect( buttonBox, SIGNAL( accepted() ), this, SLOT( accept() ) );
+      connect( buttonBox, SIGNAL( rejected() ), this, SLOT( reject() ) );
+      mainLayout->addWidget( buttonBox );
 
       KConfigGroup cg = KSharedConfig::openConfig()->group( "logdialog" );
       KWindowConfig::restoreWindowSize( windowHandle(), cg );
@@ -146,11 +138,14 @@ MainWindow::MainWindow()
              SLOT( slotOpenURLRequest( const QUrl &,
                                        const KParts::OpenUrlArguments &, const KParts::BrowserArguments & ) ) );
 
-    mNavigator = new Navigator( mDoc, mSplitter, "nav" );
+    mNavigator = new Navigator( mDoc, mSplitter );
+    mNavigator->setObjectName( "nav" );
     connect( mNavigator, SIGNAL( itemSelected( const QString & ) ),
              SLOT( viewUrl( const QString & ) ) );
     connect( mNavigator, SIGNAL( glossSelected( const GlossaryEntry & ) ),
              SLOT( slotGlossSelected( const GlossaryEntry & ) ) );
+    connect( mNavigator, SIGNAL( setStatusBarText( const QString & ) ),
+             SLOT( statusBarMessage( const QString & ) ) );
 
     mSplitter->insertWidget(0, mNavigator);
     mSplitter->setStretchFactor(mSplitter->indexOf(mNavigator), 0);
@@ -159,14 +154,12 @@ MainWindow::MainWindow()
     sizes << 220 << 580;
     mSplitter->setSizes(sizes);
 
-    KSharedConfig::Ptr cfg = KSharedConfig::openConfig();
     {
-      KConfigGroup configGroup( cfg, "General" );
-      if ( configGroup.readEntry( "UseKonqSettings", true) ) {
+      if ( Prefs::useKonqSettings() ) {
         KConfig konqCfg( "konquerorrc" );
         const_cast<KHTMLSettings *>( mDoc->settings() )->init( &konqCfg );
       }
-      const int fontScaleFactor = configGroup.readEntry( "Font zoom factor", 100 );
+      const int fontScaleFactor = Prefs::fontzoomfactor();
       mDoc->setFontScaleFactor( fontScaleFactor );
     }
 
@@ -175,6 +168,8 @@ MainWindow::MainWindow()
     foreach (QAction *act, mDoc->actionCollection()->actions())
         actionCollection()->addAction(act->objectName(), act);
 
+    setupBookmarks();
+
     setupGUI(QSize(800, 600), ToolBar | Keys | StatusBar | Create);
     setAutoSaveSettings();
 
@@ -213,8 +208,7 @@ void MainWindow::readProperties( const KConfigGroup &config )
 
 void MainWindow::readConfig()
 {
-    KConfigGroup config(KSharedConfig::openConfig(), "MainWindowState");
-    QList<int> sizes = config.readEntry( "Splitter", QList<int>() );
+    QList<int> sizes = Prefs::splitter();
     if ( sizes.count() == 2 ) {
         mSplitter->setSizes( sizes );
     }
@@ -224,8 +218,7 @@ void MainWindow::readConfig()
 
 void MainWindow::writeConfig()
 {
-    KConfigGroup config(KSharedConfig::openConfig(), "MainWindowState");
-    config.writeEntry( "Splitter", mSplitter->sizes() );
+    Prefs::setSplitter( mSplitter->sizes() );
 
     mNavigator->writeConfig();
 
@@ -291,6 +284,22 @@ void MainWindow::setupActions()
     connect( action, SIGNAL( triggered() ), this, SLOT( slotDecFontSizes() ) );
 }
 
+void MainWindow::setupBookmarks()
+{
+    const QString location = QStandardPaths::writableLocation( QStandardPaths::DataLocation );
+    QDir().mkpath( location );
+    const QString file = location + QStringLiteral( "/bookmarks.xml" );
+
+    KBookmarkManager *manager = KBookmarkManager::managerForFile( file, QStringLiteral( "khelpcenter" ) );
+    manager->setParent( this );
+    BookmarkOwner *owner = new BookmarkOwner( mDoc, manager );
+    connect( owner, SIGNAL( openUrl( const QUrl & ) ), this, SLOT( openUrl( const QUrl & ) ) );
+    KActionMenu *actmenu = actionCollection()->add<KActionMenu>( QStringLiteral( "bookmarks" ) );
+    actmenu->setText( i18nc( "@title:menu", "&Bookmarks" ) );
+    KBookmarkMenu *bookmenu = new KBookmarkMenu( manager, owner, actmenu->menu(), actionCollection() );
+    bookmenu->setParent( owner );
+}
+
 void MainWindow::slotCopySelectedText()
 {
   mDoc->copySelectedText();
@@ -320,7 +329,7 @@ void MainWindow::slotOpenURLRequest( const QUrl &url,
                                      const KParts::OpenUrlArguments &args,
                                      const KParts::BrowserArguments &browserArgs )
 {
-  qCDebug(category) << url.url();
+  khcDebug() << url.url();
 
   mNavigator->selectItem( url );
   viewUrl( url, args, browserArgs );
@@ -419,8 +428,8 @@ void MainWindow::slotGlossSelected(const GlossaryEntry &entry)
 {
     stop();
     History::self().createEntry();
-    mDoc->begin( QUrl( "help:/khelpcenter/glossary" ) );
-    mDoc->write( Glossary::entryToHtml( entry ) );
+    mDoc->begin( QUrl( "glossentry:" + entry.id() ) );
+    mDoc->write( mDoc->grantleeFormatter()->formatGlossaryEntry( entry ) );
     mDoc->end();
 }
 
@@ -486,9 +495,8 @@ void MainWindow::updateFontScaleActions()
   actionCollection()->action( QLatin1String("incFontSizes") )->setEnabled( mDoc->fontScaleFactor() + mDoc->fontScaleStepping() <= 300 );
   actionCollection()->action( QLatin1String("decFontSizes") )->setEnabled( mDoc->fontScaleFactor() - mDoc->fontScaleStepping() >= 20 );
 
-  KConfigGroup configGroup( KSharedConfig::openConfig(), QLatin1String("General") );
-  configGroup.writeEntry( QLatin1String("Font zoom factor"), mDoc->fontScaleFactor() );
-  configGroup.sync();
+  Prefs::setFontzoomfactor( mDoc->fontScaleFactor() );
+  Prefs::self()->save();
 }
 
 void MainWindow::slotConfigureFonts()
